#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
pyspamd.py - v0.0.1 - 2009.10.24

Author : Jan Goebel - goebel@pi-one.net
Licence : GPL

Usage:
	# import
	import pyspamd
	# init connection settings
	pyspamd.init_unix_socket("/var/run/spamd.socket")
	# process mail
	result = pyspamd.process_email(message=email.as_string(), command="PROCESS")

Command:
	CHECK 		- Just check if the passed message is spam or not
	SYMBOLS		- Check if message is spam or not, and return score plus list
                  	  of symbols hit
	REPORT		- Check if message is spam or not, and return score plus report
	REPORT_IFSPAM	- Check if message is spam or not, and return score plus report
                  	  if the message is spam
	PROCESS		- Process this message as described above and return modified
                          message
"""

############################################################################
# Imports
############################################################################

import socket
import types

############################################################################
# Global Variables
############################################################################

_spamd_use_Socket = None
_spamd_Socket = "/var/run/spamd.socket"
_spamd_recv_Size = 1024
_spamd_command_List = ['CHECK', 'SYMBOLS', 'REPORT', 'REPORT_IFSPAM', 'SKIP', 'PROCESS']

############################################################################

def init_unix_socket(solocation="/var/run/spamd.socket"):
	"""
	Initialize pyspamd to use spamd unix local socket

	solocation (string): spamd file for local unix socket
	return: Nothing
	"""
	global _spamd_use_Socket
	global _spamd_Socket

	if type(solocation)!=types.StringType:
		raise TypeError, 'socket location should be a string not "%s"' % (solocation)

	_spamd_use_Socket = "UNIX"
	_spamd_Socket = solocation
	return

############################################################################

def process_email(message, command="PROCESS"):
	"""
	Scan email message with spamd

	message (string): complete email as string
	return: spamd result as string
	"""

	if len(message)<=0:
		raise SpamdError, 'no email message supplied'

	if command not in _spamd_command_List:
		raise SpamdError, 'unsupported command "%s"' % (command)

	spamdSocket = __init_socket__()

	spamdSocket.send("%s SPAMC/1.2\n" % (command))
	spamdSocket.send("Content-length: %s\n" % (len(message)+1))
	spamdSocket.send("\r\n ")
	spamdSocket.send(message)
	spamdSocket.shutdown(socket.SHUT_WR)

	spamdResult = ""
	recvData = spamdSocket.recv(_spamd_recv_Size)
	spamdResult += recvData
	while recvData:
		recvData = spamdSocket.recv(_spamd_recv_Size)
		spamdResult += recvData
	spamdSocket.close()

	if spamdResult.count("Content-length:")>0:
		spamdResult = spamdResult[spamdResult.find("\n", spamdResult.find("Content-length")):].strip()
	return spamdResult

############################################################################

def __init_socket__():
	"""
	create socket connected to spamd
	return: connected unix socket
	"""

	if _spamd_use_Socket=="UNIX":
		s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
		try:
			s.connect(_spamd_Socket)
		except socket.error:
			raise SpamdError, 'could not reach spamd using unix socket (%s)' % (_spamd_Socket)
	else:
		raise SpamdError, 'could not reach spamd: connection not initialized'
	return s
